#!/bin/bash

# Read disks
DRIVE_NUMBER=$(df -h | grep "20M" | grep "/dev/" | cut -f1 -d"2" | sed 's/dev//g' | sed 's/\///g' | sed 's/[0-9]*//g' | head -n1 | xargs)
DRIVE_PARTITION=$DRIVE_NUMBER"1"
DRIVE_PARTITION2=$DRIVE_NUMBER"4"
DRIVE_EFI=$DRIVE_NUMBER"3"
if [ "$DRIVE_NUMBER" = "nvmenp" ]; then
  echo "Changing header, NVM drive detected.."
  DRIVE_NUMBER="$(df -h | grep "20M" | grep "/dev/" | cut -f1 -d"2" | sed 's/dev//g' | sed 's/\///g' | xargs | sed 's/.$//' | sed 's/\s.*$//' | xargs | sed 's/\p//g')"
  DRIVE_PARTITION=$DRIVE_NUMBER"p1"
  DRIVE_EFI=$DRIVE_NUMBER"p3"
fi

# Version management
version=$(cat /etc/lsb-release | grep "DISTRIB_RELEASE=" | sed 's/[^.0-9]*//g' | head -c 3)
version_r=$(cat /etc/lsb-release | grep "DISTRIB_RELEASE=" | sed 's/[^0-9]*//g' | head -c 3)

# Change target partition for v1.9 package or higher
# Ubuntu 20.04 || Ubuntu 22.04 base
if [[ "$version_r" -gt "189" ]]; then
  # Fix NVME
  if [[ "$DRIVE_NUMBER" == *nvme* ]]; then
    echo "v190 Partition map - NVME"
    DRIVE_NUMBER="$(df -h | grep "20M" | grep "/dev/" | cut -f1 -d"2" | sed 's/dev//g' | sed 's/\///g' | xargs | sed 's/.$//' | sed 's/\s.*$//' | xargs | sed 's/\p//g')"
    DRIVE_PARTITION=$DRIVE_NUMBER"p2"
    DRIVE_EFI=$DRIVE_NUMBER"p3"
  else
    echo "v190 Partition map - SATA"
    DRIVE_PARTITION=$DRIVE_NUMBER"2"
  fi
fi

# Compare partitions
PARTITION_MAX_SIZE_IN_BYTE=$(sudo cat /proc/partitions | grep $DRIVE_NUMBER | head -n1 | awk '{print $3}')
#PARTITION_MAX_SIZE_IN_MB=$(python -c "print $PARTITION_MAX_SIZE_IN_BYTE / 1024")
PARTITION_MAX_SIZE_IN_MB=$(python3 -c "print($PARTITION_MAX_SIZE_IN_BYTE / 1024)")
CURRENT_PARTITION_SIZE_IN_BYTE=$(sudo cat /proc/partitions | grep $DRIVE_PARTITION | head -n1 | awk '{print $3}')
CURRENT_PARTITION_SIZE_IN_MB=$(python3 -c "print($CURRENT_PARTITION_SIZE_IN_BYTE / 1024)")
SIZE_DIFFERENCE=$(python3 -c "print($PARTITION_MAX_SIZE_IN_MB - $CURRENT_PARTITION_SIZE_IN_MB)" | cut -f1 -d".")
#CURRENT_PARTITION_SIZE_IN_MB=$(python -c "print $CURRENT_PARTITION_SIZE_IN_BYTE / 1024")
#SIZE_DIFFERENCE=$(python -c "print $PARTITION_MAX_SIZE_IN_MB - $CURRENT_PARTITION_SIZE_IN_MB" | cut -f1 -d".") # 0.1 x 1000 = 100Mb

echo "-*- Expanding /dev/$DRIVE_PARTITION Partition -*-"

#echo $PARTITION_MAX_SIZE_IN_BYTE
echo "Drive max size: $PARTITION_MAX_SIZE_IN_MB Mb"
#echo $CURRENT_PARTITION_SIZE_IN_GB
echo "Current max size: $CURRENT_PARTITION_SIZE_IN_MB Mb"
echo "Difference in size: $SIZE_DIFFERENCE Mb"

# Keep 300Mb difference between drive and partition size for check
if [ "$SIZE_DIFFERENCE" -lt "300" ]; then
  RESIZED="RESIZED"
else
  RESIZED="NEED"
fi

if [ "$RESIZED" = "RESIZED" ]; then
  echo "=== ALREADY RESIZED ==="
  # Safety Check
  CURRENT_FREE_SPACE_IN_MB="$(df -hm | grep $DRIVE_PARTITION | awk '{print $4}')"
  if [ "$CURRENT_FREE_SPACE_IN_MB" -lt "285" ]; then
    # 285 Mb less free space try resize anyway
    sudo resize2fs /dev/$DRIVE_PARTITION
  fi
else
  if [[ "$version_r" -gt "189" ]]; then
    # e.g: sda2
    if [[ "$DRIVE_NUMBER" == *nvme* ]]; then
      DRIVE_PARTITION=$DRIVE_NUMBER"p2"
      sudo growpart "/dev/$DRIVE_NUMBER" "2"
      sudo resize2fs "/dev/$DRIVE_PARTITION"
    else
      DRIVE_PARTITION=$DRIVE_NUMBER"2"
      sudo growpart "/dev/$DRIVE_NUMBER" 2
      sudo resize2fs "/dev/$DRIVE_PARTITION"
    fi
  else
    echo "=== RESIZING ==="
    # Live extend
    sudo growpart "/dev/$DRIVE_NUMBER" 4
    sudo growpart "/dev/$DRIVE_NUMBER" 1
    sudo resize2fs /dev/$DRIVE_PARTITION
    sudo resize2fs /dev/$DRIVE_PARTITION2
  fi
  echo ""
  CURRENT_FREE_SPACE_IN_MB="$(df -hm | grep $DRIVE_PARTITION | awk '{print $4}')"
  echo "Free Space on the Disk: $CURRENT_FREE_SPACE_IN_MB Mb"
fi

# Clean close
exit 1
